# Copyright 2022 FeatureProbe
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

def defaultable(cls):
    _defaults = {}

    def check_or_create(*args, **kwargs):
        if args or kwargs:
            return cls(*args, **kwargs)
        if cls not in _defaults:
            default = cls()
            _defaults[cls] = default
            return default
        return _defaults[cls]

    return check_or_create
